@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* test.c: test builtin */
-/* (C) 2007-2012 magicant */
+/* (C) 2007-2018 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,10 +27,18 @@
 #include <unistd.h>
 #include <wchar.h>
 #include <wctype.h>
+#if YASH_ENABLE_DOUBLE_BRACKET
+# include "../expand.h"
+#endif
+#include "../option.h"
+#if YASH_ENABLE_DOUBLE_BRACKET
+# include "../parser.h"
+#endif
 #include "../path.h"
 #include "../plist.h"
 #include "../strbuf.h"
 #include "../util.h"
+#include "../xfnmatch.h"
 
 
 #define Exit_TRUE      0  /* Exit_SUCCESS */
@@ -57,10 +65,6 @@ static bool test_long_and(struct test_state *state)
     __attribute__((nonnull));
 static bool test_long_term(struct test_state *state)
     __attribute__((nonnull));
-static bool is_unary_primary(const wchar_t *word)
-    __attribute__((nonnull,pure));
-static bool is_binary_primary(const wchar_t *word)
-    __attribute__((nonnull,pure));
 static bool is_term_delimiter(const wchar_t *word)
     __attribute__((nonnull,pure));
 static int compare_integers(const wchar_t *left, const wchar_t *right)
@@ -70,6 +74,21 @@ static int compare_versions(const wchar_t *left, const wchar_t *right)
 static enum filecmp compare_files(const wchar_t *left, const wchar_t *right)
     __attribute__((nonnull));
 
+#if YASH_ENABLE_DOUBLE_BRACKET
+static int eval_dbexp(const dbexp_T *e)
+    __attribute__((nonnull));
+static inline wchar_t *expand_double_bracket_operand(const wordunit_T *w)
+    __attribute__((nonnull,malloc,warn_unused_result));
+static wchar_t *expand_and_unescape_double_bracket_operand(const wordunit_T *w)
+    __attribute__((nonnull,malloc,warn_unused_result));
+static bool test_triple_db(
+	const wchar_t *lhs, const wchar_t *op, const wchar_t *rhs_escaped)
+    __attribute__((nonnull));
+static bool test_triple_args(
+	const wchar_t *left, const wchar_t *op, const wchar_t *right)
+    __attribute__((nonnull));
+#endif
+
 
 /* The "test" ("[") built-in. */
 int test_builtin(int argc, void **argv)
@@ -145,6 +164,11 @@ bool test_double(void *args[static 2])
 		int fd;
 		return xwcstoi(arg, 10, &fd) && isatty(fd);
 	    }
+	case L'o':
+	    if (arg[0] == L'?')
+		return is_valid_option_name(&arg[1]);
+	    else
+		return option_is_enabled(arg);
     }
 
     char *mbsarg = malloc_wcstombs(arg);
@@ -237,6 +261,8 @@ bool test_triple(void *args[static 3])
 		return wcscmp(left, right) == 0;
 	    if (op[1] == L'=' && op[2] == L'=' && op[3] == L'\0')
 		return wcscoll(left, right) == 0;
+	    if (op[1] == L'~' && op[2] == L'\0')
+		return match_regex(left, right);
 	    goto not_binary;
 	case L'!':
 	    if (op[1] == L'=' && op[2] == L'\0')
@@ -445,8 +471,8 @@ bool is_unary_primary(const wchar_t *word)
     switch (word[1]) {
 	case L'b':  case L'c':  case L'd':  case L'e':  case L'f':  case L'G':
 	case L'g':  case L'h':  case L'k':  case L'L':  case L'N':  case L'n':
-	case L'O':  case L'p':  case L'r':  case L'S':  case L's':  case L't':
-	case L'u':  case L'w':  case L'x':  case L'z':
+	case L'O':  case L'o':  case L'p':  case L'r':  case L'S':  case L's':
+	case L't':  case L'u':  case L'w':  case L'x':  case L'z':
 	    return true;
 	default:
 	    return false;
@@ -459,7 +485,7 @@ bool is_binary_primary(const wchar_t *word)
 {
     switch (word[0]) {
 	case L'=':
-	    if (word[1] == L'\0')
+	    if (word[1] == L'\0' || (word[1] == L'~' && word[2] == L'\0'))
 		return true;
 	    /* falls thru! */
 	case L'!':
@@ -678,5 +704,134 @@ const char test_syntax[] = Ngt(
 );
 #endif
 
+#if YASH_ENABLE_DOUBLE_BRACKET
+
+/* Executes the double-bracket command, evaluating the conditional expression.
+ * Returns the exit status. Prints an error message on error. */
+int exec_double_bracket(const command_T *c)
+{
+    assert(c->c_type == CT_BRACKET);
+
+    yash_error_message_count = 0;
+    return eval_dbexp(c->c_dbexp);
+}
+
+/* Evaluates the expression of a double-bracket command.
+ * You must reset `yash_error_message_count' before calling this function.
+ * Returns the exit status. Prints an error message on error. */
+int eval_dbexp(const dbexp_T *e)
+{
+    int lhs_result;
+    bool result;
+    wchar_t *lhs = NULL, *rhs = NULL;
+
+    switch (e->type) {
+	case DBE_OR:
+	    lhs_result = eval_dbexp(e->lhs.subexp);
+	    if (lhs_result != Exit_FALSE)
+		return lhs_result;
+	    return eval_dbexp(e->rhs.subexp);
+	case DBE_AND:
+	    lhs_result = eval_dbexp(e->lhs.subexp);
+	    if (lhs_result != Exit_TRUE)
+		return lhs_result;
+	    return eval_dbexp(e->rhs.subexp);
+	case DBE_NOT:
+	    switch (eval_dbexp(e->rhs.subexp)) {
+		case Exit_TRUE:   return Exit_FALSE;
+		case Exit_FALSE:  return Exit_TRUE;
+		default:          return Exit_TESTERROR;
+	    }
+
+	case DBE_UNARY:
+	    rhs = expand_and_unescape_double_bracket_operand(e->rhs.word);
+	    if (rhs == NULL)
+		return Exit_TESTERROR;
+	    result = test_double((void *[]) { e->operator, rhs });
+	    break;
+	case DBE_BINARY:
+	    lhs = expand_and_unescape_double_bracket_operand(e->lhs.word);
+	    if (lhs == NULL)
+		return Exit_TESTERROR;
+	    rhs = expand_double_bracket_operand(e->rhs.word);
+	    if (rhs == NULL)
+		return Exit_TESTERROR;
+	    result = test_triple_db(lhs, e->operator, rhs);
+	    break;
+	case DBE_STRING:
+	    rhs = expand_and_unescape_double_bracket_operand(e->rhs.word);
+	    if (rhs == NULL)
+		return Exit_TESTERROR;
+	    result = test_single((void *[]) { rhs });
+	    break;
+
+	default:
+	    assert(false);
+    }
+
+    free(lhs);
+    free(rhs);
+    if (yash_error_message_count > 0)
+	return Exit_TESTERROR;
+    return result ? Exit_TRUE : Exit_FALSE;
+}
+
+/* Expands the operand of a primary.
+ * The result may contain backslash escapes. */
+wchar_t *expand_double_bracket_operand(const wordunit_T *w)
+{
+    return expand_single(w, TT_SINGLE, true, false);
+}
+
+/* Expands the operand of a primary.
+ * The result is literal (does not contain backslash escapes). */
+wchar_t *expand_and_unescape_double_bracket_operand(const wordunit_T *w)
+{
+    wchar_t *e = expand_double_bracket_operand(w);
+    if (e == NULL)
+	return NULL;
+    return unescapefree(e);
+}
+
+/* Tests the specified three-token (binary) primary in the double-bracket
+ * command. The left-hand-side must be given literal while the right-hand-side
+ * backslash-escaped. */
+bool test_triple_db(
+	const wchar_t *lhs, const wchar_t *op, const wchar_t *rhs_escaped)
+{
+    /* Some string comparison primaries in the double-bracket command are
+     * different from those in the test built-in. */
+    switch (op[0]) {
+	case L'=':
+	    if (op[1] == L'~') {
+		assert(op[2] == L'\0');
+		return test_triple_args(lhs, op, rhs_escaped);
+	    }
+	    if (op[1] == L'\0' || (op[1] == L'=' && op[2] == L'\0'))
+		return match_pattern(lhs, rhs_escaped);
+	    break;
+	case L'!':
+	    assert(op[1] == L'=');
+	    if (op[2] == L'\0')
+		return !match_pattern(lhs, rhs_escaped);
+	    break;
+    }
+
+    wchar_t *rhs = unescape(rhs_escaped);
+    bool result = test_triple_args(lhs, op, rhs);
+    free(rhs);
+    return result;
+}
+
+/* Tests the specified three-token expression. */
+bool test_triple_args(
+	const wchar_t *left, const wchar_t *op, const wchar_t *right)
+{
+    void *args[] = { (void *) left, (void *) op, (void *) right, };
+    return test_triple(args);
+}
+
+#endif /* YASH_ENABLE_DOUBLE_BRACKET */
+
 
 /* vim: set ts=8 sts=4 sw=4 noet tw=80: */
