@@ -57,6 +57,9 @@
 #include "variable.h"
 #include "xfnmatch.h"
 #include "yash.h"
+#if YASH_ENABLE_DOUBLE_BRACKET
+# include "builtins/test.h"
+#endif
 #if YASH_ENABLE_LINEEDIT
 # include "lineedit/complete.h"
 # include "lineedit/lineedit.h"
@@ -193,6 +196,8 @@ int laststatus = Exit_SUCCESS;
 int savelaststatus = -1;  // -1 if not in a trap handler
 /* exit status of the last command substitution */
 static int lastcmdsubstatus;
+/* exit status of the command that immediately preceded the EXIT trap. */
+int exitstatus = -1;  // -1 if not executing the EXIT trap
 /* the process ID of the last asynchronous list */
 pid_t lastasyncpid;
 
@@ -516,14 +521,8 @@ void exec_case(const command_T *c, bool finally_exit)
 	    if (pattern == NULL)
 		goto fail;
 
-	    xfnmatch_T *xfnm = xfnm_compile(
-		    pattern, XFNM_HEADONLY | XFNM_TAILONLY);
+	    bool match = match_pattern(word, pattern);
 	    free(pattern);
-	    if (xfnm == NULL)
-		continue;
-
-	    bool match = (xfnm_wmatch(xfnm, word).start != (size_t) -1);
-	    xfnm_free(xfnm);
 	    if (match) {
 		if (ci->ci_commands != NULL) {
 		    exec_and_or_lists(ci->ci_commands, finally_exit);
@@ -708,6 +707,9 @@ bool is_err_condition_for(const command_T *c)
     switch (c->c_type) {
 	case CT_SIMPLE:
 	case CT_SUBSHELL:
+#if YASH_ENABLE_DOUBLE_BRACKET
+	case CT_BRACKET:
+#endif
 	case CT_FUNCDEF:
 	    return true;
 	case CT_GROUP:
@@ -782,7 +784,6 @@ fail:
 pid_t exec_process(
 	command_T *restrict c, exec_T type, pipeinfo_T *restrict pi, pid_t pgid)
 {
-    bool early_fork;   /* do early fork? */
     bool finally_exit; /* never return? */
     int argc;
     void **argv = NULL;
@@ -793,16 +794,16 @@ pid_t exec_process(
 
     /* fork first if `type' is E_ASYNC, the command type is subshell,
      * or there is a pipe. */
-    early_fork = (type != E_SELF)
-	&& (type == E_ASYNC || c->c_type == CT_SUBSHELL
-		|| pi->pi_fromprevfd >= 0 || pi->pi_tonextfds[PIPE_OUT] >= 0);
     finally_exit = (type == E_SELF);
-    if (early_fork) {
-	sigtype_T sigtype = (type == E_ASYNC) ? t_quitint : 0;
-	cpid = fork_and_reset(pgid, type == E_NORMAL, sigtype);
-	if (cpid != 0)
-	    goto done;
-	finally_exit = true;
+    if (!finally_exit) {
+	if (type == E_ASYNC || c->c_type == CT_SUBSHELL
+		|| pi->pi_fromprevfd >= 0 || pi->pi_tonextfds[PIPE_OUT] >= 0) {
+	    sigtype_T sigtype = (type == E_ASYNC) ? t_quitint : 0;
+	    cpid = fork_and_reset(pgid, type == E_NORMAL, sigtype);
+	    if (cpid != 0)
+		goto done;
+	    finally_exit = true;
+	}
     }
 
     lastcmdsubstatus = Exit_SUCCESS;
@@ -901,7 +902,8 @@ pid_t exec_process(
 
     /* create a child process to execute the external command */
     if (cmdinfo.type == CT_EXTERNALPROGRAM && !finally_exit) {
-	cpid = fork_and_reset(pgid, type == E_NORMAL, t_leave);
+	assert(type == E_NORMAL);
+	cpid = fork_and_reset(pgid, true, t_leave);
 	if (cpid != 0)
 	    goto done3;
 	finally_exit = true;
@@ -1034,6 +1036,7 @@ pid_t fork_and_reset(pid_t pgid, bool fg, sigtype_T sigtype)
 	clear_shellfds(sigtype & t_leave);
 	is_interactive_now = false;
 	suppresserrreturn = false;
+	exitstatus = -1;
     }
     return cpid;
 }
@@ -1189,6 +1192,13 @@ void exec_nonsimple_command(command_T *c, bool finally_exit)
     case CT_CASE:
 	exec_case(c, finally_exit);
 	break;
+#if YASH_ENABLE_DOUBLE_BRACKET
+    case CT_BRACKET:
+	laststatus = exec_double_bracket(c);
+	if (finally_exit)
+	    exit_shell();
+	break;
+#endif /* YASH_ENABLE_DOUBLE_BRACKET */
     case CT_FUNCDEF:
 	exec_funcdef(c, finally_exit);
 	break;
