@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* exec.h: command execution */
-/* (C) 2007-2016 magicant */
+/* (C) 2007-2018 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ typedef enum sigtype_T {
 #define Exit_ASSGNERR Exit_ERROR
 #define Exit_REDIRERR Exit_ERROR
 
-extern int laststatus, savelaststatus;
+extern int laststatus, savelaststatus, exitstatus;
 extern pid_t lastasyncpid;
 extern _Bool special_builtin_executed;
 extern _Bool is_executing_auxiliary;
