@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* display.h: display control */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,10 +55,8 @@ extern void le_display_flush(void);
 extern void le_display_update(_Bool cursor);
 extern void le_display_make_rawvalues(void);
 extern void le_display_complete_cleanup(void);
-extern size_t le_display_select_column(int offset)
-    __attribute__((pure));
-extern size_t le_display_select_page(int offset)
-    __attribute__((pure));
+extern void le_display_select_column(int offset);
+extern void le_display_select_page(int offset);
 
 extern _Bool le_try_print_prompt(const wchar_t *s)
     __attribute__((nonnull));
