@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* variable.c: deals with shell variables and parameters */
-/* (C) 2007-2017 magicant */
+/* (C) 2007-2018 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -302,8 +302,7 @@ void init_variables(void)
 
     /* set $PS1~4 */
     {
-	const wchar_t *ps1 =
-	    !posixly_correct ? L"\\$ " : (geteuid() != 0) ? L"$ " : L"# ";
+	const wchar_t *ps1 = (geteuid() != 0) ? L"$ " : L"# ";
 	set_variable(L VAR_PS1, xwcsdup(ps1),   SCOPE_GLOBAL, false);
 	set_variable(L VAR_PS2, xwcsdup(L"> "), SCOPE_GLOBAL, false);
 	set_variable(L VAR_PS4, xwcsdup(L"+ "), SCOPE_GLOBAL, false);
@@ -768,7 +767,10 @@ bool do_assignments(const assign_T *assign, bool temp, bool export)
 void xtrace_variable(const wchar_t *name, const wchar_t *value)
 {
     xwcsbuf_T *buf = get_xtrace_buffer();
-    wb_wprintf(buf, L" %ls=%ls", name, value);
+    wb_wccat(buf, L' ');
+    wb_cat(buf, name);
+    wb_wccat(buf, L'=');
+    wb_quote_as_word(buf, value);
 }
 
 /* Pushes a trace of the specified array assignment to the xtrace buffer. */
@@ -779,7 +781,7 @@ void xtrace_array(const wchar_t *name, void *const *values)
     wb_wprintf(buf, L" %ls=(", name);
     if (*values != NULL) {
 	for (;;) {
-	    wb_cat(buf, *values);
+	    wb_quote_as_word(buf, *values);
 	    values++;
 	    if (*values == NULL)
 		break;
@@ -1293,6 +1295,9 @@ void hash_all_commands_recursively(const command_T *c)
 	    case CT_CASE:
 		hash_all_commands_in_case(c->c_casitems);
 		break;
+#if YASH_ENABLE_DOUBLE_BRACKET
+	    case CT_BRACKET:
+#endif
 	    case CT_FUNCDEF:
 		break;
 	}
@@ -1613,6 +1618,7 @@ const struct xgetopt_T typeset_options[] = {
 #endif
     { L'\0', NULL, 0, false, NULL, },
 };
+/* Note: `local_options' is defined as part of `typeset_options'. */
 
 /* The "typeset" built-in, which accepts the following options:
  *  -f: affect functions rather than variables
@@ -1623,6 +1629,7 @@ const struct xgetopt_T typeset_options[] = {
  *  -X: cancel exportation of variables
  * Equivalent built-ins:
  *  export:   typeset -gx
+ *  local:    typeset
  *  readonly: typeset -gr
  * The "set" built-in without any arguments is redirected to this built-in. */
 int typeset_builtin(int argc, void **argv)
@@ -1630,9 +1637,11 @@ int typeset_builtin(int argc, void **argv)
     bool function = false, global = false, print = false;
     bool readonly = false, export = false, unexport = false;
 
+    const struct xgetopt_T *options =
+	(ARGV(0)[0] == L'l' /*local*/) ? local_options : typeset_options;
     const struct xgetopt_T *opt;
     xoptind = 0;
-    while ((opt = xgetopt(argv, typeset_options, 0)) != NULL) {
+    while ((opt = xgetopt(argv, options, 0)) != NULL) {
 	switch (opt->shortopt) {
 	    case L'f':  function = true;  break;
 	    case L'g':  global   = true;  break;
@@ -1649,18 +1658,32 @@ int typeset_builtin(int argc, void **argv)
 	}
     }
 
-    if (wcscmp(ARGV(0), L"export") == 0) {
-	global = true;
-	if (!unexport)
-	    export = true;
-    } else if (wcscmp(ARGV(0), L"readonly") == 0) {
-	global = readonly = true;
-    } else {
-	assert(wcscmp(ARGV(0), L"typeset") == 0
-	    || wcscmp(ARGV(0), L"set") == 0);
+    switch (ARGV(0)[0]) {
+	case L'e':
+	    assert(wcscmp(ARGV(0), L"export") == 0);
+	    global = true;
+	    if (!unexport)
+		export = true;
+	    break;
+	case L'l':
+	    assert(wcscmp(ARGV(0), L"local") == 0);
+	    break;
+	case L'r':
+	    assert(wcscmp(ARGV(0), L"readonly") == 0);
+	    global = readonly = true;
+	    break;
+	case L's':
+	    assert(wcscmp(ARGV(0), L"set") == 0);
+	    global = true;
+	    break;
+	case L't':
+	    assert(wcscmp(ARGV(0), L"typeset") == 0);
+	    break;
+	default:
+	    assert(false);
     }
 
-    if (function && global && wcscmp(ARGV(0), L"typeset") == 0)
+    if (function && global && ARGV(0)[0] == L't' /*typeset*/)
 	return special_builtin_error(
 		mutually_exclusive_option_error(L'f', L'g'));
     if (function && export)
@@ -1775,7 +1798,7 @@ void print_variable(
 	return;
 
     if (!is_name(name))
-	name = qname = quote_sq(name);
+	name = qname = quote_as_word(name);
 
     switch (var->v_type & VF_MASK) {
 	case VF_SCALAR:
@@ -1790,8 +1813,9 @@ void print_variable(
 }
 
 /* Prints the specified scalar variable to the standard output.
- * If `is_name(name)' is not true, the quoted name must be given as `qname';
- * otherwise, `qname' must be NULL.
+ * `namequote' must equal `is_name(name)'. If `argv0' is "set" and `namequote'
+ * is true, the variable is not printed since it cannot be assigned in the
+ * normal assignment syntax.
  * An error message is printed to the standard error on error. */
 void print_scalar(const wchar_t *name, bool namequote,
 	const variable_T *var, const wchar_t *argv0)
@@ -1801,7 +1825,7 @@ void print_scalar(const wchar_t *name, bool namequote,
     xstrbuf_T opts;
 
     if (var->v_value != NULL)
-	quotedvalue = quote_sq(var->v_value);
+	quotedvalue = quote_as_word(var->v_value);
     else
 	quotedvalue = NULL;
     switch (argv0[0]) {
@@ -1817,8 +1841,12 @@ void print_scalar(const wchar_t *name, bool namequote,
 	    format = (quotedvalue != NULL) ? "%ls %ls=%ls\n" : "%ls %ls\n";
 	    xprintf(format, argv0, name, quotedvalue);
 	    break;
+	case L'l':
+	    assert(wcscmp(argv0, L"local") == 0);
+	    goto typeset;
 	case L't':
 	    assert(wcscmp(argv0, L"typeset") == 0);
+typeset:
 	    sb_init(&opts);
 	    if (var->v_type & VF_EXPORT)
 		sb_ccat(&opts, 'x');
@@ -1847,7 +1875,7 @@ void print_array(
 	return;
     if (var->v_valc > 0) {
 	for (size_t i = 0; ; ) {
-	    wchar_t *qvalue = quote_sq(var->v_vals[i]);
+	    wchar_t *qvalue = quote_as_word(var->v_vals[i]);
 	    bool ok = xprintf("%ls", qvalue);
 	    free(qvalue);
 	    if (!ok)
@@ -1874,8 +1902,12 @@ void print_array(
 		    || wcscmp(argv0, L"readonly") == 0);
 	    xprintf("%ls %ls\n", argv0, name);
 	    break;
+	case L'l':
+	    assert(wcscmp(argv0, L"local") == 0);
+	    goto typeset;
 	case L't':
 	    assert(wcscmp(argv0, L"typeset") == 0);
+typeset:
 	    sb_init(&opts);
 	    if (var->v_type & VF_EXPORT)
 		sb_ccat(&opts, 'x');
@@ -1904,7 +1936,7 @@ void print_function(
 
     wchar_t *qname = NULL;
     if (!is_name(name))
-	name = qname = quote_sq(name);
+	name = qname = quote_as_word(name);
 
     wchar_t *value = command_to_wcs(func->f_body, true);
     const char *format = (qname == NULL) ? "%ls()\n%ls" : "function %ls()\n%ls";
@@ -1945,6 +1977,12 @@ const char export_help[] = Ngt(
 const char export_syntax[] = Ngt(
 "\texport [-prX] [name[=value]...]\n"
 );
+const char local_help[] = Ngt(
+"set or print local variables"
+);
+const char local_syntax[] = Ngt(
+"\tlocal [-prxX] [name[=value]...]\n"
+);
 const char readonly_help[] = Ngt(
 "make variables read-only"
 );
